@@ -38,6 +38,8 @@ use Lintian::Data ();
 
 my $BUGS_NUMBER
   = Lintian::Data->new('changelog-file/bugs-number', qr/\s*=\s*/o);
+my $INVALID_DATES
+  = Lintian::Data->new('changelog-file/invalid-dates', qr/\s*=\>\s*/o);
 
 my $SPELLING_ERROR_IN_NEWS
   = spelling_tag_emitter('spelling-error-in-news-debian');
@@ -171,7 +173,7 @@ sub run {
                 m/^changelog\.debian(?:\.gz)$/i;
             }
             @doc_files
-          ) {
+        ) {
             tag 'wrong-name-for-changelog-of-native-package',
               "usr/share/doc/$pkg/$chg";
         } else {
@@ -187,19 +189,13 @@ sub run {
             # everything is fine
         } else {
             # search for changelogs with wrong file name
-            my $found = 0;
             for (@doc_files) {
                 if (m/^change/i and not m/debian/i) {
                     tag 'wrong-name-for-upstream-changelog',
                       "usr/share/doc/$pkg/$_";
-                    $found = 1;
                     last;
                 }
             }
-            if (not $found) {
-                tag 'no-upstream-changelog'
-                  unless $info->is_pkg_class('transitional');
-            }
         }
 
         # 2. check for Debian changelog
@@ -210,7 +206,7 @@ sub run {
                 m/^changelog\.debian(?:\.gz)?$/i;
             }
             @doc_files
-          ) {
+        ) {
             tag 'wrong-name-for-debian-changelog-file',
               "usr/share/doc/$pkg/$chg";
         } else {
@@ -278,7 +274,15 @@ sub run {
         }
 
         if ($first_timestamp) {
+            my $warned = 0;
             my $dch_date = $entries[0]->Date;
+            foreach my $re ($INVALID_DATES->all()) {
+                if ($dch_date =~ m/($re)/i) {
+                    my $repl = $INVALID_DATES->value($re);
+                    tag 'invalid-date-in-debian-changelog', "($1 -> $repl)";
+                    $warned = 1;
+                }
+            }
             my ($weekday_declared, $date) = split(m/,\s*/, $dch_date, 2);
             $date //= '';
             my ($tz, $weekday_actual);
@@ -287,7 +291,7 @@ sub run {
                 $tz = $1;
                 $weekday_actual = time2str('%a', $first_timestamp, $tz);
             }
-            if ($tz and $weekday_declared ne $weekday_actual) {
+            if (not $warned and $tz and $weekday_declared ne $weekday_actual) {
                 my $real_weekday = time2str('%A', $first_timestamp, $tz);
                 my $short_date = time2str('%Y-%m-%d', $first_timestamp, $tz);
                 tag 'debian-changelog-has-wrong-day-of-week',
@@ -299,7 +303,7 @@ sub run {
             my $second_timestamp = $entries[1]->Timestamp;
 
             if ($first_timestamp && $second_timestamp) {
-                tag 'latest-debian-changelog-entry-without-new-date'
+                tag 'latest-changelog-entry-without-new-date'
                   unless (($first_timestamp - $second_timestamp) > 0
                     or lc($entries[0]->Distribution) eq 'unreleased');
             }
@@ -309,28 +313,43 @@ sub run {
             if ($first_version and $second_version) {
                 tag 'latest-debian-changelog-entry-without-new-version'
                   unless versions_gt($first_version, $second_version)
-                  or $entries[0]->Changes =~ /backport/i;
+                  or $entries[0]->Changes =~ /backport/i
+                  or $entries[0]->Source ne $entries[1]->Source;
                 tag 'latest-debian-changelog-entry-changed-to-native'
                   if $native_pkg and $second_version =~ m/-/;
             }
+            my $first_version_without_epoch = $first_version =~ s/^([^:]+)://r;
+            foreach my $entry (@entries[1..$#entries]) {
+                my $version_without_epoch = $entry->Version =~ s/^([^:]+)://r;
+                if (    $first_version_without_epoch eq $version_without_epoch
+                    and $entries[0]->Source eq $entry->Source) {
+                    tag
+                      'latest-debian-changelog-entry-reuses-existing-version',
+                      "$first_version == $entry->{Version}",
+                      "(last used: $entry->{Date})";
+                    last;
+                }
+            }
 
             my $first_upstream = $first_version;
             $first_upstream =~ s/-[^-]+$//;
             my $second_upstream = $second_version;
             $second_upstream =~ s/-[^-]+$//;
-            my $first_debian = $first_version;
-            $first_debian =~ s/^[^-]+-//;
-            my $second_debian = $second_version;
-            $second_debian =~ s/^[^-]+-//;
+            my $first_debian =substr $first_version, length($first_upstream);
+            $first_debian =~ s/-([^-]+)$/$1/ if length($first_debian) > 0;
+            my $second_debian =substr $second_version,length($second_upstream);
+            $second_debian =~ s/-([^-]+)$/$1/ if length($second_debian) > 0;
 
-            if ($first_upstream eq $second_upstream) {
+            if (    $first_upstream eq $second_upstream
+                and $entries[0]->Source eq $entries[1]->Source) {
                 if ($entries[0]->Changes
                     =~ /^\s*\*\s+new\s+upstream\s+(?:\S+\s+)?release\b/im) {
                     tag 'possible-new-upstream-release-without-new-version';
                 }
                 if ($first_debian =~ /^\d+$/ and $second_debian =~ /^\d+$/) {
                     unless ($first_debian == $second_debian + 1) {
-                        tag 'non-consecutive-debian-revision';
+                        tag 'non-consecutive-debian-revision',
+                          "$first_version -> $second_version";
                     }
                 }
             }
@@ -343,6 +362,21 @@ sub run {
                     tag 'experimental-to-unstable-without-comment';
                 }
             }
+
+            my ($first_epoch) = ($first_version =~ /^([^:]+):/, '(none)');
+            my ($second_epoch) = ($second_version =~ /^([^:]+):/, '(none)');
+            if ($first_epoch and $second_epoch ne $first_epoch) {
+                tag 'epoch-change-without-comment',
+                  "$second_epoch -> $first_epoch"
+                  unless $entries[0]->Changes =~ /\bepoch\b/im;
+
+                my ($first_no_epoch) = ($first_upstream =~ s/^([^:]+)://r);
+                my ($second_no_epoch) = ($second_upstream =~ s/^([^:]+)://r);
+                tag
+                  'epoch-changed-but-upstream-version-did-not-go-backwards',
+                  "$first_no_epoch >= $second_no_epoch"
+                  unless versions_gt($second_no_epoch, $first_no_epoch);
+            }
         }
 
         # Some checks should only be done against the most recent
@@ -371,7 +405,7 @@ sub run {
         if (
             $changes =~ /uploads? \s+ to \s+
                             (?'intended'testing|unstable|experimental|sid)/xi
-          ){
+        ){
             my $intended = lc($+{intended});
             if($intended eq 'sid') {
                 $intended = 'unstable';
@@ -460,9 +494,12 @@ sub run {
         # length check.
         my @lines = split("\n", decode('utf-8', $changes));
         for my $i (0 .. $#lines) {
+            my $line = $i + $chloff;
+            tag 'debian-changelog-line-too-short', $1, "(line $line)"
+              if $lines[$i] =~ /^   [*]\s(.{1,5})$/ and $1 !~ /:$/;
             if (length($lines[$i]) > 81
                 and $lines[$i] !~ /^[\s.o*+-]*(?:[Ss]ee:?\s+)?\S+$/) {
-                tag 'debian-changelog-line-too-long', 'line ' . ($i + $chloff);
+                tag 'debian-changelog-line-too-long', "line $line";
             }
         }
 
@@ -505,7 +542,7 @@ sub check_dch {
             || m/closes:\s*(?:bug)?\#?\s?\d+
               (?:,\s*(?:bug)?\#?\s?\d+)*
               (?:,\s*(((?:bug)?\#?\s?\d*)[[:alpha:]]\w*))/iox
-          ) {
+        ) {
             tag 'wrong-bug-number-in-closes', "l$.:$1" if $2;
         }
 
