@@ -43,6 +43,7 @@ use constant {
 };
 
 our $KNOWN_ESSENTIAL = Lintian::Data->new('fields/essential');
+our $KNOWN_TOOLCHAIN = Lintian::Data->new('fields/toolchain');
 our $KNOWN_METAPACKAGES = Lintian::Data->new('fields/metapackages');
 our $NO_BUILD_DEPENDS = Lintian::Data->new('fields/no-build-depends');
 our $KNOWN_SECTIONS = Lintian::Data->new('fields/archive-sections');
@@ -51,9 +52,30 @@ our $known_build_essential
 our $KNOWN_BINARY_FIELDS = Lintian::Data->new('fields/binary-fields');
 our $KNOWN_UDEB_FIELDS = Lintian::Data->new('fields/udeb-fields');
 our $KNOWN_BUILD_PROFILES = Lintian::Data->new('fields/build-profiles');
+our $KNOWN_VCS_HOSTERS= Lintian::Data->new(
+    'fields/vcs-hosters',
+    qr/\s*~~\s*/,
+    sub {
+        my @ret = split(',', $_[1]);
+        return \@ret;
+    });
+my $KNOWN_INSECURE_HOMEPAGE_URIS
+  = Lintian::Data->new('fields/insecure-homepage-uris');
+my $DERIVATIVE_VERSIONS= Lintian::Data->new('fields/derivative-versions',
+    qr/\s*~~\s*/, sub { $_[1]; });
 
 our %KNOWN_ARCHIVE_PARTS = map { $_ => 1 } ('non-free', 'contrib');
 
+my $DERIVATIVE_FIELDS = Lintian::Data->new(
+    'fields/derivative-fields',
+    qr/\s*\~\~\s*/,
+    sub {
+        my ($regexp, $explanation) = split(/\s*\~\~\s*/, $_[1], 2);
+        return {
+            'regexp' => qr/$regexp/,
+            'explanation' => $explanation,
+        };
+    });
 my $KNOWN_PRIOS = Lintian::Data->new('fields/priorities');
 
 our @supported_source_formats = (qr/1\.0/, qr/3\.0\s*\((quilt|native)\)/);
@@ -103,6 +125,9 @@ my $NAME_SECTION_MAPPINGS = Lintian::Data->new(
         return {'regex' =>  qr/$_[0]/x, 'section' => $_[1]};
     });
 
+our $DH_ADDONS = Lintian::Data->new('common/dh_addons', '=');
+our %DH_ADDONS_VALUES = map { $DH_ADDONS->value($_) => 1 } $DH_ADDONS->all;
+
 my %VCS_EXTRACT = (
     browser => sub { return @_;},
     arch    => sub { return @_;},
@@ -111,11 +136,12 @@ my %VCS_EXTRACT = (
     cvs     => sub { return shift =~ /^(.+?)(?:\s+(\S*))?$/;},
     darcs   => sub { return @_;},
     hg      => sub { return @_;},
-    # git uri followed by optional " -b " + branchname:
-    git     => sub { return shift =~ /^(.+?)(?:\s+-b\s+(\S*))?$/;},
+    # git uri followed by optional "[subdir]", "-b branchname" etc.
+    git     =>
+      sub { return shift =~ /^(.+?)(?:\s+\[(\S*)\])?(?:\s+-b\s+(\S*))?$/;},
     svn     => sub { return @_;},
-    # that's a hostname followed by a module name:
-    mtn     => sub { return shift =~ /^(.+?)\s+(\S+)$/;},
+    # New "mtn://host?branch" uri or deprecated "host branch".
+    mtn     => sub { return shift =~ /^(.+?)(?:\s+\S+)?$/;},
 );
 my %VCS_CANONIFY = (
     browser => sub {
@@ -125,6 +151,9 @@ my %VCS_CANONIFY = (
                   {https://anonscm.debian.org/git/};
         $_[0] =~ s{https?\Q://bzr.debian.org/loggerhead/\E}
                   {https://anonscm.debian.org/loggerhead/};
+        $_[0] =~ s{https?\Q://salsa.debian.org/\E([^/]+/[^/]+)\.git/?$}
+                  {https://salsa.debian.org/$1};
+
         if ($_[0] =~ m{https?\Q://anonscm.debian.org/viewvc/\E}xsm) {
             if ($_[0] =~ s{\?(.*[;\&])?op=log(?:[;\&](.*))?\Z}{}xsm) {
                 my (@keep) = ($1, $2, $3);
@@ -138,7 +167,7 @@ my %VCS_CANONIFY = (
         if (
             $_[0] =~ s{\@(?:cvs\.alioth|anonscm)\.debian\.org:/cvsroot/}
                       {\@anonscm.debian.org:/cvs/}
-          ) {
+        ) {
             $_[1] = 'vcs-field-bitrotted';
         }
         $_[0]=~ s{\@\Qcvs.alioth.debian.org:/cvs/}{\@anonscm.debian.org:/cvs/};
@@ -157,7 +186,7 @@ my %VCS_CANONIFY = (
         if (
             $_[0] =~ s{git://(?:git|anonscm)\.debian\.org/~}
                       {https://anonscm.debian.org/git/users/}
-          ) {
+        ) {
             $_[1] = 'vcs-git-uses-invalid-user-uri';
         }
         $_[0] =~ s{https?\Q://git.debian.org/\E(?:git/?)?}
@@ -168,6 +197,8 @@ my %VCS_CANONIFY = (
                   {https://anonscm.debian.org/git/};
         $_[0] =~ s{\Qgit://anonscm.debian.org/git/\E}
                   {https://anonscm.debian.org/git/};
+        $_[0] =~ s{https?\Q://salsa.debian.org/\E([^/]+/[^/\.]+)(?!\.git)$}
+                  {https://salsa.debian.org/$1.git};
     },
     hg      => sub {
         $_[0] =~ s{https?\Q://hg.debian.org/\E}
@@ -195,7 +226,7 @@ my %VCS_RECOMMENDED_URIS = (
     hg      => qr;^https?://;,
     git     => qr;^(?:git|https?|rsync)://;,
     svn     => qr;^(?:svn|(?:svn\+)?https?)://;,
-    mtn     => qr;^[\w.-]+$;,
+    mtn     => qr;^mtn://;,
 );
 my %VCS_VALID_URIS = (
     arch    => qr;^https?://;,
@@ -204,6 +235,7 @@ my %VCS_VALID_URIS = (
     hg      => qr;^ssh://;,
     git     => qr;^(?:git\+)?ssh://|^[\w.]+@[a-zA-Z0-9.]+:[/a-zA-Z0-9.];,
     svn     => qr;^(?:svn\+)?ssh://;,
+    mtn     => qr;^[\w.-]+$;,
 );
 
 # Python development packages that are used almost always just for building
@@ -249,6 +281,7 @@ sub run {
             unfold('package', \$name);
             tag 'bad-package-name' unless $name =~ /^$PKGNAME_REGEX$/i;
             tag 'package-not-lowercase' if ($name =~ /[A-Z]/);
+            tag 'unusual-documentation-package-name' if $name =~ /-docs$/;
         }
     }
 
@@ -278,7 +311,7 @@ sub run {
                     && $debian =~ m/\A
                             (?:[^.]+ubuntu[^.]+)(?:\.\d+){1,3}(\..*)?
                     \Z/oxsm
-                  ) {
+                ) {
                     $ubuntu = 1;
                     $extra = $1;
                 }
@@ -300,7 +333,7 @@ sub run {
                     (rc|alpha|beta|pre(?:view|release)?)
                    ([^a-z].*|\Z)
                  /xsm
-              ) {
+            ) {
                 my $expected = $upstream;
                 my $rc = $1;
                 my $rest = $2//'';
@@ -327,6 +360,15 @@ sub run {
                 }
             }
 
+            if (!$info->native) {
+                foreach my $re ($DERIVATIVE_VERSIONS->all) {
+                    next if $version =~ m/$re/;
+                    my $explanation = $DERIVATIVE_VERSIONS->value($re);
+                    tag 'invalid-version-number-for-derivative', $version,
+                      "($explanation)";
+                }
+            }
+
             my $name = $info->field('package');
             if (   $name
                 && $PERL_CORE_PROVIDES->known($name)
@@ -353,10 +395,21 @@ sub run {
             $arch =~ s/\n//o;
             tag 'illegal-multi-arch-value', $arch, $march if ($arch eq 'all');
         }
+
+    }
+
+    if ($type eq 'source') {
+        for my $bin ($info->binaries) {
+            my $arch = $info->binary_field($bin, 'architecture');
+            my $fname = "debian/$bin.lintian-overrides.$arch";
+            next unless $info->binary_field($bin, 'multi-arch', '') eq 'same';
+            tag 'multi-arch-same-package-has-arch-specific-overrides', $fname
+              if $info->index_resolved_path($fname);
+        }
     }
 
     if ($type eq 'binary'){
-        if ($pkg =~ /^fonts-/) {
+        if ($pkg =~ /^x?fonts-/) {
             tag 'font-package-not-multi-arch-foreign'
               unless $info->field('multi-arch', 'no')
               =~ m/^(?:foreign|allowed)$/o;
@@ -402,6 +455,15 @@ sub run {
                 if ($pkg =~ /-docs?$/ && $archs[0] ne 'all') {
                     tag 'documentation-package-not-architecture-independent';
                 }
+                if ($pkg =~ /^r-(?:cran|bioc|other)-/ and $archs[0] ne 'all') {
+                    for my $file ($info->sorted_index) {
+                        next if $file->is_dir;
+                        next if $file !~ m,^usr/lib/R/.*/DESCRIPTION,;
+                        tag 'r-package-not-arch-all'
+                          if $file->file_contents =~ m/NeedsCompilation: no/m;
+                        last;
+                    }
+                }
             }
         }
         # Used for later tests.
@@ -417,12 +479,16 @@ sub run {
     #---- Maintainer
     #---- Uploaders
 
+    my $is_comaintained = 0;
     for my $f (qw(maintainer uploaders)) {
         if (not defined $info->field($f)) {
             tag 'no-maintainer-field' if $f eq 'maintainer';
         } else {
             my $maintainer = $info->field($f);
 
+            my $is_list = $maintainer =~ /\@lists(?:\.alioth)?\.debian\.org\b/;
+            $is_comaintained = 1 if $is_list;
+
             # Note, not expected to hit on uploaders anymore, as dpkg
             # now strips newlines for the .dsc, and the newlines don't
             # hurt in debian/control
@@ -438,6 +504,7 @@ sub run {
                 my @uploaders = map { split /\@\S+\K\s*,\s*/ }
                   split />\K\s*,\s*/, $maintainer;
                 for my $uploader (@uploaders) {
+                    $is_comaintained = 1;
                     check_maintainer($uploader, 'uploader');
                     if (   ((true { $_ eq $uploader } @uploaders) > 1)
                         and($duplicate_uploaders{$uploader}++ == 0)) {
@@ -447,7 +514,7 @@ sub run {
             } else {
                 check_maintainer($maintainer, $f);
                 if (   $type eq 'source'
-                    && $maintainer =~ /\@lists(?:\.alioth)?\.debian\.org\b/
+                    && $is_list
                     && !defined $info->field('uploaders')) {
                     tag 'no-human-maintainers';
                 }
@@ -486,7 +553,7 @@ sub run {
             \s*
             # Optional Version e.g. (1.0)
             (?:\((?:\d+:)?(?:[-\.+:a-zA-Z0-9~]+?)(?:-[\.+a-zA-Z0-9~]+)?\))?\s*$/x
-          ) {
+        ) {
             tag 'source-field-malformed', $source;
         }
     }
@@ -514,7 +581,9 @@ sub run {
 
         unfold('section', \$section);
 
-        if ($type eq 'udeb') {
+        if ($section eq '') {
+            tag 'empty-section-field';
+        } elsif ($type eq 'udeb') {
             unless ($section eq 'debian-installer') {
                 tag 'wrong-section-for-udeb', $section;
             }
@@ -588,6 +657,13 @@ sub run {
 
         tag 'unknown-priority', $priority
           unless $KNOWN_PRIOS->known($priority);
+
+        tag 'excessive-priority-for-library-package', $priority
+          if $pkg =~ m/^lib/o
+          and $pkg !~ m/-bin$/o
+          and $pkg !~ m/^libc[0-9.]+$/o
+          and any { $_ eq $info->field('section', '') } qw(libdevel libs)
+          and any { $_ eq $priority } qw(required important standard);
     }
 
     #---- Standards-Version
@@ -626,6 +702,11 @@ sub run {
         if ($homepage=~ m,bioconductor\.org/packages/.*/bioc/html/.*\.html*$,){
             tag 'homepage-for-bioconductor-package-not-canonical', $orig;
         }
+        foreach my $re ($KNOWN_INSECURE_HOMEPAGE_URIS->all) {
+            next unless $homepage =~ m/$re/;
+            tag 'homepage-field-uses-insecure-uri', $orig;
+            last;
+        }
     } elsif (not $info->native) {
         if ($type eq 'source') {
             my $binary_has_homepage_field = 0;
@@ -681,7 +762,7 @@ sub run {
         $javalib = 1 if($pkg =~ m/^lib.*-java$/o);
         for my $field (
             qw(depends pre-depends recommends suggests conflicts provides enhances replaces breaks)
-          ) {
+        ) {
             next unless defined $info->field($field);
             #Get data and clean it
             my $data = $info->field($field);
@@ -699,6 +780,7 @@ sub run {
 
             tag 'alternates-not-allowed', $field
               if ($data =~ /\|/ && !&$is_dep_field($field));
+            check_field($info, $field, $data) if &$is_dep_field($field);
 
             for my $dep (split /\s*,\s*/, $data) {
                 my (@alternatives, @seen_obsolete_packages);
@@ -759,7 +841,8 @@ sub run {
                       if ($field eq 'conflicts' && $d_version->[0]);
 
                     tag 'obsolete-relation-form', "$field: $part_d_orig"
-                      if ($d_version && any { $d_version->[0] eq $_ }
+                      if (
+                        $d_version && any { $d_version->[0] eq $_ }
                         ('<', '>'));
 
                     tag 'bad-version-in-relation', "$field: $part_d_orig"
@@ -860,6 +943,14 @@ sub run {
                         && $pkg !~ m/-(?:dev|docs?|tools|bin)$/
                         && $part_d_orig =~ m/-docs?$/);
 
+                    tag 'binary-package-depends-on-toolchain-package',
+                      "$field: $part_d_orig"
+                      if $KNOWN_TOOLCHAIN->known($d_pkg)
+                      and &$is_dep_field($field)
+                      and not $pkg =~ m/^dh-/
+                      and not $pkg =~ m/-(source|src)$/
+                      and not $DH_ADDONS_VALUES{$pkg};
+
                     # default-jdk-doc must depend on openjdk-X-doc (or
                     # classpath-doc) to be useful; other packages
                     # should depend on default-jdk-doc if they want
@@ -985,11 +1076,12 @@ sub run {
         my %depend;
         for my $field (
             qw(build-depends build-depends-indep build-depends-arch build-conflicts build-conflicts-indep build-conflicts-arch)
-          ) {
+        ) {
             if (defined $info->field($field)) {
                 #Get data and clean it
                 my $data = $info->field($field);
                 unfold($field, \$data);
+                check_field($info, $field, $data);
                 $depend{$field} = $data;
 
                 for my $dep (split /\s*,\s*/, $data) {
@@ -1054,7 +1146,7 @@ sub run {
                           if ($d_pkg eq 'build-essential');
 
                         #<<< no tidy, tag name too long
-                        tag 'depends-on-build-essential-package-without-using-version',
+                        tag 'build-depends-on-build-essential-package-without-using-version',
                         #>>>
                           "$d_pkg [$field: $part_d_orig]"
                           if ($known_build_essential->known($d_pkg)
@@ -1144,7 +1236,7 @@ sub run {
             $depend{'build-conflicts'},
             $depend{'build-conflicts-indep'},
             $depend{'build-conflicts-arch'}
-          ) {
+        ) {
             next unless $_;
             for my $conflict (split /\s*,\s*/, $_) {
                 if ($build_all->implies($conflict)) {
@@ -1185,22 +1277,20 @@ sub run {
             tag 'build-depends-on-python-dev-with-no-arch-any';
         }
 
+        my $bdepends = $info->relation('build-depends');
+
         # libmodule-build-perl
         # matches() instead of implies() because of possible OR relation
-        if ($info->relation('build-depends-indep')
-            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)
-            && !$info->relation('build-depends')
-            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)) {
-            tag 'libmodule-build-perl-needs-to-be-in-build-depends';
-        }
+        tag 'libmodule-build-perl-needs-to-be-in-build-depends'
+          if $info->relation('build-depends-indep')
+          ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)
+          && !$bdepends->matches(qr/^libmodule-build-perl$/,VISIT_PRED_NAME);
 
         # libmodule-build-tiny-perl
-        if ($info->relation('build-depends-indep')
-            ->implies('libmodule-build-tiny-perl')
-            && !$info->relation('build-depends')
-            ->implies('libmodule-build-tiny-perl')) {
-            tag 'libmodule-build-tiny-perl-needs-to-be-in-build-depends';
-        }
+        tag 'libmodule-build-tiny-perl-needs-to-be-in-build-depends'
+          if $info->relation('build-depends-indep')
+          ->implies('libmodule-build-tiny-perl')
+          && !$bdepends->implies('libmodule-build-tiny-perl');
     }
 
     #----- Origin
@@ -1218,47 +1308,10 @@ sub run {
 
         tag 'redundant-bugs-field'
           if $bugs =~ m,^debbugs://bugs.debian.org/?$,i;
-    }
 
-    #----- Python-Version
-
-    if (defined(my $pyversion = $info->field('python-version'))) {
-        unfold('python-version', \$pyversion);
-
-        my @valid = (
-            ['\d+\.\d+', '\d+\.\d+'],['\d+\.\d+'],
-            ['\>=\s*\d+\.\d+', '\<\<\s*\d+\.\d+'],['\>=\s*\d+\.\d+'],
-            ['current', '\>=\s*\d+\.\d+'],['current'],
-            ['all']);
-
-        my @pyversion = split(/\s*,\s*/, $pyversion);
-
-        if ($pyversion =~ m/^current/) {
-            tag 'python-version-current-is-deprecated';
-        }
-
-        if (@pyversion > 2) {
-            if (any { !/^\d+\.\d+$/ } @pyversion) {
-                tag 'malformed-python-version', $pyversion;
-            }
-        } else {
-            my $okay = 0;
-            for my $rule (@valid) {
-                if (
-                    $pyversion[0] =~ /^$rule->[0]$/
-                    && ((
-                               $pyversion[1]
-                            && $rule->[1]
-                            && $pyversion[1] =~ /^$rule->[1]$/
-                        )
-                        || (!$pyversion[1] && !$rule->[1]))
-                  ) {
-                    $okay = 1;
-                    last;
-                }
-            }
-            tag 'malformed-python-version', $pyversion unless $okay;
-        }
+        tag 'bugs-field-does-not-refer-to-debian-infrastructure', $bugs,
+          "(line $.)"
+          unless $bugs =~ m,\.debian\.org, or $pkg =~ /[-]dbgsym$/;
     }
 
     #----- Dm-Upload-Allowed
@@ -1275,6 +1328,7 @@ sub run {
 
     #----- Vcs-*
 
+    my %seen_vcs;
     while (my ($vcs, $splitter) = each %VCS_EXTRACT) {
         if (defined $info->field("vcs-$vcs")) {
             my $uri = $info->field("vcs-$vcs");
@@ -1312,16 +1366,69 @@ sub run {
                     tag $tag, $parts[0], $canonicalized;
                 }
             }
-            tag 'vcs-browser-links-to-empty-view', $uri
-              if $vcs eq 'browser' and $uri =~ m%rev=0&sc=0%;
+            if ($vcs eq 'browser') {
+                tag 'vcs-browser-links-to-empty-view', $uri
+                  if $uri =~ m%rev=0&sc=0%;
+            } else {
+                $seen_vcs{$vcs}++;
+                foreach my $regex ($KNOWN_VCS_HOSTERS->all) {
+                    foreach my $re_vcs (@{$KNOWN_VCS_HOSTERS->value($regex)}) {
+                        if (   $uri =~ m/^($regex.*)/xi
+                            && $vcs ne $re_vcs
+                            && $vcs ne 'browser') {
+                            tag 'vcs-field-mismatch',
+                              "vcs-$vcs != vcs-$re_vcs",
+                              $uri;
+                            last;
+                        }
+                    }
+                }
+            }
+            if ($uri =~ m{//(.+)\.debian\.org/}) {
+                tag 'vcs-deprecated-in-debian-infrastructure', "vcs-$vcs", $uri
+                  unless $1 =~ m{^(?:salsa|.*\.dgit)$};
+            } else {
+                tag 'orphaned-package-not-maintained-in-debian-infrastructure',
+                  "vcs-$vcs", $uri
+                  if $info->field('maintainer', '')
+                  =~ /packages\@qa.debian.org/
+                  and $vcs ne 'browser';
+            }
+        }
+    }
+    tag 'vcs-fields-use-more-than-one-vcs', sort keys %seen_vcs
+      if keys %seen_vcs > 1;
+
+    tag 'co-maintained-package-with-no-vcs-fields'
+      if $type eq 'source'
+      and $is_comaintained
+      and not %seen_vcs;
+
+    # Check for missing Vcs-Browser headers
+    if (!defined $info->field('vcs-browser')) {
+        foreach my $regex ($KNOWN_VCS_HOSTERS->all) {
+            my $vcs = @{$KNOWN_VCS_HOSTERS->value($regex)}[0];
+            if ($info->field("vcs-$vcs", '') =~ m/^($regex.*)/xi) {
+                tag 'missing-vcs-browser-field', "vcs-$vcs", $1;
+                last; # Only warn once
+            }
         }
     }
 
     #---- Checksums
 
+    tag 'no-strong-digests-in-dsc'
+      if $type eq 'source' && !$info->field('checksums-sha256');
+
+    #----- Derivative-specific field checks
+
     if ($type eq 'source') {
-        if (!$info->field('checksums-sha256')) {
-            tag 'no-strong-digests-in-dsc';
+        foreach my $field ($DERIVATIVE_FIELDS->all) {
+            my $val = $info->field($field, '-');
+            my $data = $DERIVATIVE_FIELDS->value($field);
+            tag 'invalid-field-for-derivative',
+              "$field: $val ($data->{'explanation'})"
+              if $val !~ m/$data->{'regexp'}/;
         }
     }
 
@@ -1406,6 +1513,30 @@ sub unfold {
     return;
 }
 
+sub check_field {
+    my ($info, $field, $data) = @_;
+
+    my $has_default_mta
+      = $info->relation($field)->matches(qr/^default-mta$/, VISIT_PRED_NAME);
+    my $has_mail_transport_agent = $info->relation($field)
+      ->matches(qr/^mail-transport-agent$/, VISIT_PRED_NAME);
+
+    tag 'default-mta-dependency-not-listed-first',"$field: $data"
+      if $info->relation($field)
+      ->matches(qr/\|\s+default-mta/, VISIT_OR_CLAUSE_FULL);
+
+    if ($has_default_mta) {
+        tag 'default-mta-dependency-does-not-specify-mail-transport-agent',
+          "$field: $data"
+          unless $has_mail_transport_agent;
+    } elsif ($has_mail_transport_agent) {
+        tag 'mail-transport-agent-dependency-does-not-specify-default-mta',
+          "$field: $data"
+          unless $has_default_mta;
+    }
+    return;
+}
+
 1;
 
 # Local Variables:
