@@ -191,7 +191,7 @@ sub run {
             not $initd_path
             or (    not $info->is_conffile($initd_path->name)
                 and not $initd_path->is_symlink)
-          ) {
+        ) {
             tag 'init.d-script-not-marked-as-conffile',
               "etc/init.d/$initd_file";
         }
@@ -200,6 +200,7 @@ sub run {
         # other issues if it was included in the package.
         check_init($initd_path, $info);
     }
+    check_defaults($info);
 
     return unless $initd_dir and $initd_dir->is_dir;
 
@@ -255,6 +256,10 @@ sub check_init {
                 }
             }
         }
+        if ($l =~ m/Please remove the "Author" lines|Example initscript/) {
+            tag 'init.d-script-contains-skeleton-template-content',
+              "${initd_path}:$.";
+        }
         if ($l =~ m/^\#\#\# BEGIN INIT INFO/) {
             if ($lsb{BEGIN}) {
                 tag 'init.d-script-has-duplicate-lsb-section',$initd_path;
@@ -321,8 +326,9 @@ sub check_init {
             $tag{$1} = 1;
         }
 
-        if ($l =~ m{^\s*\.\s+/lib/lsb/init-functions}
-            && !$info->relation('strong')->implies('lsb-base')) {
+        if (   $l =~ m{^\s*\.\s+/lib/lsb/init-functions}
+            && !$info->relation('strong')->implies('lsb-base')
+            && none { $_->basename =~ m/\.service$/ } $info->sorted_index) {
             tag 'init.d-script-needs-depends-on-lsb-base',
               $initd_path, "(line $.)";
         }
@@ -432,8 +438,10 @@ sub check_init {
     if (defined $lsb{'default-stop'} && length($lsb{'default-stop'})) {
         my @required = split(' ', $lsb{'required-stop'} || '');
         if ($needs_fs) {
-            if (none { /^(?:\$(?:local|remote)_fs|\$all|umountn?fs)\z/ }
-                @required) {
+            if (
+                none { /^(?:\$(?:local|remote)_fs|\$all|umountn?fs)\z/ }
+                @required
+            ) {
                 tag 'init.d-script-missing-dependency-on-local_fs',
                   "${initd_path}: required-stop";
             }
@@ -477,6 +485,22 @@ sub check_init {
     return;
 }
 
+sub check_defaults {
+    my ($info) = @_;
+    my $dir = $info->index_resolved_path('etc/default/');
+    return unless $dir and $dir->is_dir;
+    for my $path ($dir->children) {
+        return if not $path->is_open_ok;
+        my $fd = $path->open;
+        while (<$fd>) {
+            tag 'init.d-script-should-always-start-service', $path, "(line $.)"
+              if m/^\s*#*\s*(?:ENABLED|DISABLED|[A-Z]*RUN)=/;
+        }
+        close($fd);
+    }
+    return;
+}
+
 1;
 
 # Local Variables:
